/*
   orxonox - the future of 3D-vertical-scrollers
 
   Copyright (C) 2006 orx
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.
 
   ### File Specific:
   main-programmer: bottac@ee.ethz.ch
*/

#include <vector>
#include <deque>


// FORWARD DECLARATIONS
class  BspFile;
class BspTreeLeaf;
class BspTreeNode;
class Vector;
class set;
struct face;
struct brush;
struct plane;

class WorldEntity;

class BspManager
{
public:
  // Constructors
  BspManager();

  // Functions
  void draw();
  void draw_debug_face(int Face); 
  void draw_face(int Face);
  void draw_patch(face* Face);

  void checkCollision(WorldEntity* worldEntity);

private:
  // Functions
  BspTreeNode* getLeaf(BspTreeNode*  node,   Vector* cam) ;  //!< Traverses the tree
  void  checkCollision(BspTreeNode* node, Vector* cam);
  void  checkCollisionRay(BspTreeNode * node,float startFraction, float endFraction, Vector* start, Vector* end); 
  void  checkBrushRay(brush* curBrush);
  void drawDebugCube(Vector* cam);
  bool isAlreadyVisible(int Face);
  void addFace(int Face);
  
  // Data
  BspFile*  bspFile;
  BspTreeNode* root;
  Vector cam;
  Vector ship;
  Vector  viewDir;
  plane* collPlane;
  int lastTex;
  
  //obsolete
  bool  outputStartsOut;
  bool  outputAllSolid;
  float outputFraction;
  
  bool * alreadyVisible;
  // Deques to store the visible faces
  ::std::deque<int> trasparent; //!< the ones with transparancy go here
  ::std::deque<int> opal; //!< the others here.

  Vector out;
  Vector out1;
  Vector out2;
};

